/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 10 January 2018
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#include <cmath>

/*   fitted NLO cross sections     */
static double cross_section(double R, double pT)
{
  static constexpr double nlo_r2[5][2] = {
    {-74.01658024427968, 82.27464268680053},
    {0.05068502435619231, 0.0965808934449842},
    {-0.000017605895272345978, 0.00013828272191312535},
    {2.6384566022715035e-9, 3.891301893927592e-9},
    {- 1.598226805133714e-13, 3.533428324248293e-11},
  };
  
  static constexpr double nlo_r4[5][2] = {
    {-89.43435686639582, 97.74159682256081},
    {0.06373702674018425, 0.12366319025063298},
    {- 0.00002184162264881441, 0.00015681748444247627},
    {3.26427371209164e-9, 1.3879593301841329e-8},
    {-1.9513040776090802e-13, 3.959183680137702e-11}
  };
  
  static constexpr double nlo_r7[5][2] = {
    {-70.49248257257523, 79.07855165290054},
    {0.04821580036966591, 0.09186201699778505},
    {-0.000016891480741551407, 0.00013138952672733594},
    {2.5440457443326026e-9, 4.20362419219724e-9},
    {-1.5504258764303244e-13, 3.307942245376652e-11}
  };
  
  /*    calculationg the interpolation   */
  const double (*c)[2] = (R < 0.3 ? nlo_r2 : (R < 0.6 ? nlo_r4 : nlo_r7));
  double res = 0.0, expT = std::exp(-pT/400.0);
  
  for(int k = 4; k > 0; --k)
  {
    res += c[k][0] + c[k][1]*expT;
    res *= pT;
  }
  
  return std::exp(res + c[0][0] + c[0][1]*expT);
}



/*   local headers  */
#include "analyzer-jets.h"



void UserJets::initfunc()
{
  /*   Createing histograms to calculate the one jet cross sections   */
  unsigned int nbins = 45;
  double pTmin = 287.0_GeV, pTmax = 4967.0_GeV;
  
  /*   cross sections after shower    */
  HistArr::phys(1, "One jet inclusive cross section ratios",
                spacing<hist1d>::linear(nbins, pTmin, pTmax));   // Bin width is 104 GeV
}



void UserJets::userfunc(const shower_stage<hhc>& /*  hard  */,
                        const shower_stage<hhc>& /*  afterthreshold  */,
                        const shower_stage<hhc>& showered)
{
  constexpr double tonb = 389379.338;
  
  /*   showered event   */
  auto& p = showered.state;
  
  /*   Converting deductor event to FastJet format   */
  make_pseudojets(theparticles, p);
  
  /*    Run fastjet algorithm   */
  fj::ClusterSequence clustSeq(theparticles, jetDef);
  
  /*    Extract inclusive jets sorted by pT (above pTjetmin)   */
  double pTjetmin = 200.0_GeV;
  incJets = clustSeq.inclusive_jets(pTjetmin);
  
  
  /*   This is a resonable breakdown of the full weight
   *     0 => Standard weight (no delta, no threshold)
   *     1 => Standard * (Standard Threshold)
   *     2 => (Standard + Delta) (no threshold)
   *     3 => (Standard + Delta + DeltaThreshold)*(Standard Threshold)
   *     4 => (Standard + Delta + Phase) (no threshold)
   *     5 => (Standard + Delta + Phase + DeltaThreshold)*(Standard Threshold)
   */
  std::array<double, 6> w, wi = weight_info(showered);

  
  /*    Fill histogram, only count jets that have |y| < 2.0   */
  for(auto& pj : incJets)
    if(abs(pj.pseudorapidity()) < 2.0)
    {
      double pT = pj.perp();
      double nlo = cross_section(jetDef.R(), pT);
      
      w[0] = tonb*wi[0]/nlo;
      w[1] = tonb*wi[1]/nlo;
      w[2] = tonb*wi[2]/nlo;
      w[3] = tonb*wi[3]/nlo;
      w[4] = tonb*wi[4]/nlo;
      w[5] = tonb*wi[5]/nlo;

      HistArr::physfill(1, w, dirac{}, pT);
    }
}

