/*  File: analyzer-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 18 January 2019
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-unitary.h"



void UserUnitary::initfunc()
{
  /*   Unitary test    */
  HistArr6::phys(1, "Unitarity test", spacing<hist1d>::linear(25, 0.0, 10.0));
  HistArr22::phys(1, "Unitarity test (partial contributions)", spacing<hist1d>::linear(25, 0.0, 10.0));
}



void UserUnitary::userfunc(const shower_stage<hhc>& hard,
                           const shower_stage<hhc>&,
                           const shower_stage<hhc>& showered)
{

  std::array<double, 6> weights;
  std::array<double, 22> partial;

  weights.fill(0.0);
  partial.fill(0.0);

  /*   the zeroth element is the Born cross section    */
  weights[0] = partial[0] = weight(hard);
 
  /* Note: we have
   delta = deltaH+deltaV+phase;
   phase = pahseR+phaseL+phaseU;
   */
  
  /*   showered cross section    */
  auto func = [&weights, &partial](const partial_weight_t& w)
  {
    /*    we ignore threshold corrections    */
    if(w.deltaT != 0u) return;
    
    
    /*  pure LC+ contribution    */
    if(w.delta == 0u)
    {
      weights[1] += w.wstd;
    }
    
    /*  One insertion of Delta operators   */
    else if((w.deltaH + w.deltaV) == 1u && w.phase == 0u)
    {
      weights[2] += w.wstd;
      
      if(w.deltaH == 1u && w.phase == 0u) partial[1] += w.wstd;
      if(w.deltaV == 1u && w.phase == 0u) partial[2] += w.wstd;
    }
    
    /*   Two insertions of Delta operators   */
    else if(((w.deltaH + w.deltaV) == 2u && w.phase == 0u) ||
            ((w.deltaH + w.deltaV) == 0u && w.phase == 2u))
    {
      weights[3] += (w.phase == 2u ? -w.wstd : w.wstd);

      if(w.deltaH == 2u && w.phase == 0u)                    partial[3] += w.wstd;
      if(w.deltaH == 1u && w.deltaV == 1u && w.phase == 0u)  partial[4] += w.wstd;
      if(w.deltaV == 2u && w.phase == 0u)                    partial[5] += w.wstd;
      if(w.deltaH == 0u && w.deltaV == 0u && w.phase == 2u)  partial[6] -= w.wstd;
    }
   
    /*   Three insertions of Delta operators   */
    else if(((w.deltaH + w.deltaV) == 3u && w.phase == 0u) ||
            ((w.deltaH + w.deltaV) == 1u && w.phase == 2u))
    {
      weights[4] += (w.phase == 2u ? -w.wstd : w.wstd);
      
      if(w.deltaH == 3u && w.phase == 0u)                     partial[7] += w.wstd;
      if(w.deltaH == 2u && w.deltaV == 1u && w.phase == 0u)   partial[8] += w.wstd;
      if(w.deltaH == 1u && w.deltaV == 2u && w.phase == 0u)   partial[9] += w.wstd;
      if(w.deltaV == 3u && w.phase == 0u)                     partial[10] += w.wstd;
      
      if(w.deltaH == 1u && w.phase == 2u)                     partial[11] -= w.wstd;
      if(w.deltaV == 1u && w.phase == 2u)                     partial[12] -= w.wstd;
   
    }
    
    /*   Four insertions of Delta operators   */
    else if(((w.deltaH + w.deltaV) == 4u && w.phase == 0u) ||
            ((w.deltaH + w.deltaV) == 2u && w.phase == 2u) ||
            ((w.deltaH + w.deltaV) == 0u && w.phase == 4u) )
    {
      weights[5] += (w.phase == 2u ? -w.wstd : w.wstd);
      
      if(w.deltaH == 4u && w.phase == 0u)                     partial[13] += w.wstd;
      if(w.deltaH == 3u && w.deltaV == 1u && w.phase == 0u)   partial[14] += w.wstd;
      if(w.deltaH == 2u && w.deltaV == 2u && w.phase == 0u)   partial[15] += w.wstd;
      if(w.deltaH == 1u && w.deltaV == 3u && w.phase == 0u)   partial[16] += w.wstd;
      if(w.deltaV == 4u && w.phase == 0u)                     partial[17] += w.wstd;
      
      if(w.deltaH == 2u && w.phase == 2u)                     partial[18] -= w.wstd;
      if(w.deltaH == 1u && w.deltaV == 1u && w.phase == 2u)   partial[19] -= w.wstd;
      if(w.deltaV == 2u && w.phase == 2u)                     partial[20] -= w.wstd;
           
      if(w.deltaH == 0u && w.deltaV == 0u && w.phase == 4u)   partial[21] += w.wstd;
      
  }
    
  };
  
  /*    accumulating the weights   */
  analyse_color_weights(showered, std::move(func));
  
  
  /*    fill the histogram    */
  double tmH = static_cast<const shower_history_stage<hhc>&>(hard).current_time;

  HistArr6::physfill(1, weights, dirac{}, tmH);
  HistArr22::physfill(1, partial, dirac{}, tmH);

}

